class ProbabilisticCounter(object):
    def __init__(self):
        self._set = set()

    # returns: True - value is new, False, value already included
    def add(self, value):
        hash = value.__hash__()
        if hash in self._set:
            return False
        else:
            self._set.add(hash)
            return True

    def count(self):
        return len(self._set)



from .base import AugerBaseApi
from ..exceptions import AugerException
from .review_alert_item import AugerReviewAlertItemApi
from .review_alert import AugerReviewAlertApi


class AugerEndpointPipelineApi(AugerBaseApi):
    """Auger Endpoint Pipeline API."""

    def __init__(self, ctx, object_id):
        super(AugerEndpointPipelineApi, self).__init__(
            ctx, None, None, object_id)



from .base import AugerBaseApi
from ..exceptions import AugerException
from .review_alert_item import AugerReviewAlertItemApi
from .review_alert import AugerReviewAlertApi
from .cluster_task import AugerClusterTaskApi

class AugerEndpointApi(AugerBaseApi):
    """Auger Endpoint API."""

    def __init__(self, ctx, endpoint_api, endpoint_id=None):
        super(AugerEndpointApi, self).__init__(
            ctx, endpoint_api, None, endpoint_id)

    def create(self, pipeline_id, name):
        return self._call_create({'pipeline_id': pipeline_id, 'name': name},[])

    def update(self, params):
        params['id'] = self.object_id
        return self._call_update(params)

    def update_roi(self):
        roi_names = ['review/roi/filter', 'review/roi/investment', 'review/roi/revenue']
        roi_values = []
        roi_exists = False
        for name in roi_names:
            if self.ctx.config.get(name):
                roi_exists = True

            roi_values.append(self.ctx.config.get(name))

        if roi_exists:
            res = self.rest_api.hub_client.create_endpoint_roi_validation(
                endpoint_id=self.object_id,
                expressions=roi_values,
            )
            cluster_task = AugerClusterTaskApi(self.ctx, cluster_task_id=res['data']['id'])
            cluster_task.wait_for_status(['pending', 'received', 'started', 'retry'])
            props = cluster_task.properties()
            isValid = True
            for idx, item in enumerate(props.get('result', [])):
                if not item.get('is_valid'):
                    isValid = False
                    self.ctx.error("Review ROI config parameter '%s' = '%s' is invalid. Error: %s"%(
                        roi_names[idx], roi_values[idx], item.get('error')))                    

            if isValid:
                return self._call_update({ 'id': self.object_id, 
                    'roi_filter': roi_values[0], 'roi_investment': roi_values[1], 'roi_revenue': roi_values[2],
                })

    def update_monitoring_value(self):
        return self._call_update({ 'id': self.object_id, 
            'false_positive_cost_amount': self.ctx.config.get('review/monitoring_value/false_positive_value'), 
            'false_negative_cost_amount': self.ctx.config.get('review/monitoring_value/false_negative_value'), 
            'regression_cost_over_amount': self.ctx.config.get('review/monitoring_value/regression_value_over'), 
            'regression_cost_under_amount': self.ctx.config.get('review/monitoring_value/regression_value_under'),
        })



from a2ml.api.base_a2ml import BaseA2ML
from a2ml.api.utils.show_result import show_result

class A2MLModel(BaseA2ML):
    """Contains the model operations that interact with provider."""
    def __init__(self, ctx, provider=None):
        """Initializes a new a2ml model.

        Args:
            ctx (object): An instance of the a2ml Context.
            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider from methods.

        Returns:
            A2MLModel object

        Examples:
            .. code-block:: python

                ctx = Context()
                model = A2MLModel(ctx)
        """
        super(A2MLModel, self).__init__(ctx, "model")

        self.runner = self.build_runner(ctx, provider)
        self.local_runner = lambda: self.build_runner(ctx, provider, force_local=True)

    @show_result
    def deploy(self, model_id, locally=False, review=False, provider=None,
        name=None, algorithm=None, score=None, data_path=None, metadata=None):
        """Deploy a model locally or to specified provider(s).

        Args:
            model_id (str): Model ID from the any experiment leaderboard.
            locally(bool): Deploys using a local model if True, on the Provider Cloud if False.
            review(bool): Should model support review based on actual data. The default is True.
            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.
            name (str): Friendly name for the model. Used as name for Review Endpoint
            algorithm (str): Monitored model(external provider) algorithm name.
            score (float): Monitored model(external provider) score.
            data_path (str): Data path to fit model when deploy. Return new deployed model-id
            metadata (dict): Additional parameter for the model. Used for accurcay report(report parameter)

        Returns:
            ::

                {
                    'result': True,
                    'data': {'model_id': 'A017AC8EAD094FD'}
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                model = A2MLModel(ctx).deploy(model_id='D881079E1ED14FB', name='FirstExperiment')

            .. code-block:: python

                ctx = Context()
                model = A2MLModel(ctx, 'external')
                result = model.deploy(model_id=None, name="My external model.", algorithm='RandomForest', score=0.75)
                model_id = result['data']['model_id']

        """
        return self.get_runner(locally, model_id, provider).execute_one_provider('deploy',
            model_id, locally, review, name, algorithm, score, data_path, metadata)

    @show_result
    def predict(self, model_id, filename=None, data=None, columns=None, predicted_at=None,
            threshold=None, score=False, score_true_data=None,
            output=None, no_features_in_result=None, locally=False, provider=None):
        """Predict results with new data against deployed model. Predictions are stored next to the file with data to be predicted on. The file name will be appended with suffix _predicted.

        Note:
            Use deployed model_id \n
            This method support only one provider

        Args:
            model_id(str): The deployed model id you want to use.
            filename(str): The file with data to request predictions for.
            data: array of records [[target, actual]] or Pandas DataFrame (target, actual) or dict created with Pandas DataFrame to_dict('list') method
            columns(list): list of column names if data is array of records
            predicted_at: Predict data date. Use for review of historical data.
            threshold(float): For classification models only. This will return class probabilities with response.
            score(bool): Calculate scores for predicted results.
            score_true_data(str, pandas.DataFrame, dict): Data with true values to calculate scores. If missed, target from filename used for true values.            
            output(str): Output csv file path.
            no_features_in_result(bool) : Do not return feature columns in prediction result. False by default
            locally(bool, str): Predicts using a local model with auger.ai.predict if True, on the Provider Cloud if False. If set to "docker", then docker image used to run the model
            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider set in costructor or config.

        Returns:
            if filename is not None. ::

                {
                    'result': True,
                    'data': {'predicted': 'dataset_predicted.csv'}
                }

            if filename is None and data is not None and columns is None. ::

                {
                    'result': True,
                    'data': {'predicted': [{col1: value1, col2: value2, target: predicted_value1}, {col1: value3, col2: value4, target: predicted_value2}]}
                }

            if filename is None and data is not None and columns is not None. ::

                {
                    'result': True,
                    'data': {'predicted': {'columns': ['col1', 'col2', target], 'data': [['value1', 'value2', 1], ['value3', 'value4', 0]]}}
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                rv = A2MLModel(ctx).predict(model_id, '../irises.csv')
                # if rv[provider].result is True
                # predictions are stored in rv[provider]['data']['predicted']

            .. code-block:: python

                ctx = Context()
                data = [{'col1': 'value1', 'col2': 'value2'}, {'col1': 'value3', 'col2': 'value4'}]
                rv = A2MLModel(ctx).predict(model_id, data=data)
                # if rv[provider].result is True
                # predictions are returned as rv[provider]['data']['predicted']

            .. code-block:: python

                ctx = Context()
                data = [['value1', 'value2'], ['value3', 'value4']]
                columns = ['col1', 'col2']
                rv = A2MLModel(ctx).predict(model_id, data=data)
                # if rv[provider].result is True
                # predictions are returned as rv[provider]['data']['predicted']

            .. code-block:: python
                
                # Predict locally without config files. Model will automatically downloaded if not exists.
                # To use local predict install a2ml[predict]
                ctx = Context()
                ctx.config.set('name', 'project name')
                ctx.credentials = "Json string from a2ml ui settings"

                rv = A2MLModel(ctx).predict(model_id, '../irises.csv',
                    no_features_in_result = True, locally=True)
                # if rv[provider].result is True
                # predictions are stored in rv[provider]['data']['predicted']

        """
        return self.get_runner(locally, model_id, provider).execute_one_provider('predict', filename, model_id, 
            threshold, locally, data, columns, predicted_at, output, no_features_in_result,
            score, score_true_data )

    @show_result
    def actuals(self, model_id, filename=None, data=None, columns=None, actuals_at=None, 
        actual_date_column=None, experiment_params=None, locally=False, provider=None):
        """Submits actual results(ground truths) for predictions of a deployed model. This is used to review and monitor active models.

        Note:
            It is assumed you have predictions against this model first.

            .. list-table:: actuals.csv
                :widths: 50 50 50
                :header-rows: 1

                * - predicted ( or target): predicted value. If missed - predict called automatically
                  - actual
                  - baseline_target: predicted value for baseline model (OPTIONAL)
                * - Iris-setosa
                  - Iris-setosa
                  - Iris-setosa
                * - Iris-virginica
                  - Iris-virginica
                  - Iris-virginica

            It may also contain train features to predict(if target missed), retrain model while Review and for distribution chart

            This method support only one provider

        Args:
            model_id(str): The deployed model id you want to use.
            filename(str): The file with data to request predictions for.
            data: array of records [[target, actual]] or Pandas DataFrame (target, actual) or dict created with Pandas DataFrame to_dict('list') method
            columns(list): list of column names if data is array of records
            actuals_at: Actuals date. Use for review of historical data.
            actual_date_column(str): name of column in data which contains actual date
            experiment_params(dict): parameters to calculate experiment metrics ::

                start_date(date): experiment actuals start date
                end_date(date):  experiment actuals end date
                date_col(str): column name with date


            locally(bool): Process actuals locally.
            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider set in costructor or config.

        Returns:
            ::

                {
                    'result': True,
                    'data': True
                }

            Errors. ::

                {
                    'result': False,
                    'data': 'Actual Prediction IDs not found in model predictions.'
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                A2MLModel(ctx).actuals('D881079E1ED14FB', filename=<path_to_file>/actuals.csv)

            .. code-block:: python

                ctx = Context()
                actual_records = [['predicted_value_1', 'actual_value_1'], ['predicted_value_2', 'actual_value_2']]
                columns = [target, 'actual']

                A2MLModel(ctx).actuals('D881079E1ED14FB', data=actual_records,columns=columns)

            .. code-block:: python

                ctx = Context()
                actual_records = [['predicted_value_1', 'actual_value_1'], ['predicted_value_2', 'actual_value_2']]
                columns = [target, 'actual']

                A2MLModel(ctx, "external").actuals('external_model_id', data=actual_records,columns=columns)

        """
        return self.get_runner(locally, model_id, provider).execute_one_provider('actuals', 
            model_id, filename, data, columns, actuals_at, actual_date_column, experiment_params, locally)

    @show_result
    def review_alert(self, model_id, parameters = None, locally=False, provider=None, name=None):
        """Update Review parameters.

        Args:
            model_id(str): The deployed model id you want to use.
            parameters(dict): If None, review section from config will be used.

                * active (True/False): Activate/Deactivate Review Alert
                * type (model_accuracy/feature_average_range/runtime_errors_burst)

                    - model_accuracy: Decrease in Model Accuracy: the model accuracy threshold allowed before trigger is initiated. Default threshold: 0.7. Default sensitivity: 72
                    - feature_average_range: Feature Average Out-Of-Range: Trigger an alert if average feature value during time period goes beyond the standard deviation range calculated during training period by the specified number of times or more. Default threshold: 1. Default sensitivity: 168
                    - runtime_errors_burst: Burst Of Runtime Errors: Trigger an alert if runtime error count exceeds threshold. Default threshold: 5. Default sensitivity: 1

                * threshold (float)
                * sensitivity (int): The amount of time(in hours) this metric must be at or below the threshold to trigger the alert.
                * threshold_policy (all_values/average_value/any_value)

                    - all_values: Default value. Trigger an alert when all values in sensitivity below threshold
                    - average_value: Trigger an alert when average of values in sensitivity below threshold
                    - any_value: Trigger an alert when any value in sensitivity below threshold

                * action (no/retrain/retrain_deploy)

                    - no: no action should be executed
                    - retrain: Use new predictions and actuals as test set to retrain the model.
                    - retrain_deploy: Deploy retrained model and make it active model of this endpoint.

                * notification (no/user/organization): Send message via selected notification channel.

            locally(bool): Process review locally.
            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.
            name (str): Friendly name for the model. Used as name for Review Endpoint

        Returns:
            ::

                {
                    'result': True,
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                model = A2MLModel(ctx).review_alert(model_id='D881079E1ED14FB')
        """
        return self.get_runner(locally, model_id, provider).execute_one_provider('review_alert', model_id, parameters, name)

    @show_result
    def review(self, model_id, locally=False, provider=None):
        """Review information about deployed model.

        Args:
            model_id(str): The deployed model id you want to use.
            locally(bool): Process review locally.
            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.

        Returns:
            status(str): May be : started, error, completed, retrain
            error(str): Description of error if status='error'
            accuracy(float): Average accuracy of model(based on used metric) for review sensitivity period(see config.yml)

            ::

                {
                    'result': True,
                    'data': {'status': 'completed', 'error': '', 'accuracy': 0.76}
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                result = A2MLModel(ctx).review(model_id='D881079E1ED14FB')
        """
        return self.get_runner(locally, model_id, provider).execute_one_provider('review', model_id)

    @show_result
    def undeploy(self, model_id, locally=False, provider=None):
        """Undeploy a model locally or from specified provider(s).

        Args:
            model_id (str): Model ID from any experiment leaderboard.
            locally(bool): Deploys using a local model if True, on the Provider Cloud if False. The default is False.
            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.

        Returns:
            ::

                {
                    'result': True,
                    'data': {'model_id': 'A017AC8EAD094FD'}
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                model = A2MLModel(ctx).undeploy(model_id='D881079E1ED14FB', locally=True)
        """
        return self.get_runner(locally, model_id, provider).execute_one_provider('undeploy', model_id, locally)

    @show_result
    def delete_actuals(self, model_id, with_predictions=False, begin_date=None, end_date=None, locally=False, provider=None):
        """Delete files with actuals and predcitions locally or from specified provider(s).

        Args:
            model_id (str): Model ID to delete actuals and predictions.
            with_predictions(bool):
            begin_date: Date to begin delete operations
            end_date: Date to end delete operations
            locally(bool): Delete files from local model if True, on the Provider Cloud if False. The default is False.
            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.

        Returns:
            ::

                {
                    'result': True,
                    'data': None
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                A2MLModel(ctx).delete_actuals(model_id='D881079E1ED14FB')
        """
        return self.get_runner(locally, model_id, provider).execute_one_provider('delete_actuals', model_id, with_predictions, begin_date, end_date, locally)

    @show_result
    def get_info(self, model_id, locally=False, provider=None):
        """Get information about model

        Args:
            model_id (str): Model ID to delete actuals and predictions.
            locally(bool): Get information from local model if True, on the Provider Cloud if False. The default is False.
            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.

        Returns:
            ::

                {
                    'result': True,
                    'data': {} #detailed model information
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                res = A2MLModel(ctx).get_info('D881079E1ED14FB')
        """
        return self.get_runner(locally, model_id, provider).execute_one_provider('get_info', model_id, locally)

    @show_result
    def update(self, model_id, metadata, locally=False, provider=None):
        """Update model metadata

        Args:
            model_id (str): Model ID to delete actuals and predictions.
            metadata (dict): Model metadata to update
            locally(bool): Get information from local model if True, on the Provider Cloud if False. The default is False.
            provider (str): The automl provider you wish to run. For example 'auger'. The default is None - use provider defined by model_id or set in costructor.

        Returns:
            ::

                {
                    'result': True,
                    'data': {}
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                res = A2MLModel(ctx).update('D881079E1ED14FB', {'report': {}})
        """
        return self.get_runner(locally, model_id, provider).execute_one_provider('update', model_id, metadata, locally)



from .base import AugerBaseApi
from .cluster_task import AugerClusterTaskApi


class AugerActualApi(AugerBaseApi):
    """Auger Trial API."""

    def __init__(self, ctx, pipeline_api, use_endpoint=False):
        super(AugerActualApi, self).__init__(ctx, pipeline_api)
        self.use_endpoint = use_endpoint
        if self.use_endpoint:
            self.parent_id_name = "endpoint_id"
            self._set_api_request_path("AugerEndpointActualApi")

    def create(self, records, features, actuals_at, actuals_path, actual_date_column, experiment_params):
        params = {}
        if self.use_endpoint:
            params['endpoint_id'] = self.parent_api.object_id
            #params['prediction_group_id'] = 'dffe01f4-39bd-4f7b-832f-61051f8c7e1b'
        else:
            params['pipeline_id'] = self.parent_api.object_id

        if records:
            params['actuals'] = records

        if features:
            params['actual_column_names'] = features

        if actual_date_column:
            params['actual_date_column'] = actual_date_column

        if actuals_path:
            params['actuals_path'] = actuals_path

        if actuals_at:
            params['actuals_at'] = str(actuals_at)

        if experiment_params:
            params['experiment_params'] = experiment_params

        #print(params)
        return self._call_create(
            params=params, 
            has_return_object=False)

    def delete(self, with_predictions, begin_date, end_date):
        params = {
            'with_predictions': with_predictions,
            'from': str(begin_date) if begin_date else None,
            'to': str(end_date) if end_date else None,
        }
        if self.use_endpoint:
            params['endpoint_id'] = self.parent_api.object_id
            if not self.ctx.config.get('support_model_history'):
                params['primary_only'] = True            
        else:
            params['pipeline_id'] = self.parent_api.object_id

        res = self.rest_api.call( 'delete_%ss' % self.api_request_path, params)
        if res and isinstance(res, list):
            res = res[0]

        if res:    
            cluster_task = AugerClusterTaskApi(self.ctx, cluster_task_id=res['id'])
            cluster_task.wait_for_status(['pending', 'received', 'started', 'retry'])

        return True



import types

def print_table(log, table_list, headers=None, hor_lines=True):
    if isinstance(table_list, types.GeneratorType):
        table_list = list(table_list)

    if table_list is None or len(table_list) == 0:
        return

    col_list = headers
    if not col_list:
        col_list = list(table_list[0].keys() if table_list else [])
    row_list = [col_list]  # 1st row = header
    for item in table_list:
        row_list.append([str(item.get(col) or '') for col in col_list])
    # maximun size of the col for each element
    col_size = [max(map(len, col)) for col in zip(*row_list)]
    # insert seperating line before every line, and extra one for ending.
    if hor_lines:
        for i in range(0, len(row_list) + 1)[::-1]:
            row_list.insert(i, ['-' * i for i in col_size])
    else:
        i = 1
        row_list.insert(i, ['-' * i for i in col_size])        
    # two format for each content line and each separating line
    format_str = ' | '.join(["{{:<{}}}".format(i) for i in col_size])
    format_sep = '-+-'.join(["{{:<{}}}".format(i) for i in col_size])
    for idx, item in enumerate(row_list):
        if hor_lines:
            if idx%2==0: #item[0][0] == '-':
                log(format_sep.format(*item))
            else:
                log(format_str.format(*item))
        else:
            log(format_str.format(*item))        


import click
from a2ml.api.a2ml_model import A2MLModel
from a2ml.api.utils.context import pass_context


@click.group('model', short_help='Model management')
@pass_context
def cmdl(ctx):
    """Model management"""
    ctx.setup_logger(format='')

@click.command('deploy', short_help='Deploy trained model.')
@click.argument('model-id', required=False, type=click.STRING)
@click.option('--locally', is_flag=True, default=False,
    help='Download and deploy trained model locally.')
@click.option('--review', is_flag=True, default=False,
    help='Do not support model review based on actual data.')
@click.option('--provider', '-p', type=click.Choice(['auger','azure','external']), required=False,
    help='Cloud AutoML Provider.')
@click.option('--name', '-n', required=False, type=click.STRING, 
    help='Model friendly name.Used as name for Review Endpoint')
@click.option('--algorithm', '-a', required=False, type=click.STRING, 
    help='Monitored model(external provider) algorithm name.')
@click.option('--score', '-s', required=False, type=float, 
    help='Monitored model(external provider) score.')
@click.option('--data-path', '-d', type=click.STRING, required=False,
    help='Data path to fit model when deploy. Return new deployed model-id')
@pass_context
def deploy(ctx, provider, model_id, locally, review, name, algorithm, score, data_path):
    """Deploy trained model."""
    A2MLModel(ctx, provider).deploy(model_id, locally=locally, review=review, 
        name=name, algorithm=algorithm, score=score, data_path=data_path)

@click.command('predict', short_help='Predict with deployed model.')
@click.argument('model-id', required=True, type=click.STRING)
@click.argument('filename', required=True, type=click.STRING)
@click.option('--threshold', '-t', default=None, type=float,
    help='Threshold.')
@click.option('--score', '-s', is_flag=True, default=False,
    help='Calculate scores for predicted results.')
@click.option('--score_true_path', type=click.STRING, required=False,
    help='Path to true values to calculate scores. If missed, target from filename used for true values.')
@click.option('--locally', '-l', is_flag=True, default=False,
    help='Predict locally using auger.ai.predict package.')
@click.option('--docker', is_flag=True, default=False,
    help='Predict locally using Docker image to run model.')
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
    help='Cloud AutoML Provider.')
@click.option('--output', '-o', type=click.STRING, required=False,
    help='Output csv file path.')
@pass_context
def predict(ctx, provider, filename, model_id, threshold, locally, docker, output):
    """Predict with deployed model."""
    if docker:
        locally = "docker"

    A2MLModel(ctx, provider).predict(model_id, filename=filename, 
        threshold=threshold, score=score, score_true_data=score_true_path,
        locally=locally, output=output)

@click.command('actuals', short_help='Send actual values for deployed model. Needed for review and monitoring.')
@click.argument('model-id', required=True, type=click.STRING)
@click.argument('filename', required=True, type=click.STRING)
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
help='Cloud AutoML Provider.')
@click.option('--locally', is_flag=True, default=False,
    help='Process actuals locally.')
@pass_context
def actuals(ctx, provider, filename, model_id, locally):
    """Predict with deployed model."""
    A2MLModel(ctx, provider).actuals(model_id, filename=filename, locally=locally)

@click.command('review_alert', short_help='Update Review Alert parameters from config.')
@click.argument('model-id', required=True, type=click.STRING)
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
    help='Cloud AutoML Provider.')
@click.option('--name', '-n', required=False, type=click.STRING, 
    help='Model friendly name.Used as name for Review Endpoint')
@pass_context
def review_alert(ctx, provider, model_id, name):
    """Predict with deployed model."""
    A2MLModel(ctx, provider).review_alert(model_id, name=name)

@click.command('review', short_help='Review information about deployed model.')
@click.argument('model-id', required=True, type=click.STRING)
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
    help='Cloud AutoML Provider.')
@pass_context
def review(ctx, provider, model_id):
    """Review information about deployed model."""
    result = A2MLModel(ctx, provider).review(model_id)
    ctx.log('Review information for %s: %s' % (model_id, result))

@click.command('undeploy', short_help='Undeploy trained model.')
@click.argument('model-id', required=True, type=click.STRING)
@click.option('--locally', is_flag=True, default=False,
    help='Undeploy trained model locally.')
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
    help='Cloud AutoML Provider.')
@pass_context
def undeploy(ctx, provider, model_id, locally):
    """Undeploy trained model."""
    A2MLModel(ctx, provider).undeploy(model_id, locally)

@click.command('delete_actuals', short_help='Delete files with actuals and predcitions locally or from specified provider(s).')
@click.argument('model-id', required=True, type=click.STRING)
@click.option('--with-predictions', is_flag=True, default=False,
    help='Remove predictions.')
@click.option('--begin-date', '-b', type=click.STRING, required=False,
    help='Date to begin delete operations.')
@click.option('--end-date', '-e', type=click.STRING, required=False,
    help='Date to end delete operations.')
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
help='Cloud AutoML Provider.')
@click.option('--locally', is_flag=True, default=False,
    help='Process actuals locally.')
@pass_context
def delete_actuals(ctx, model_id, with_predictions, begin_date, end_date, provider, locally):
    A2MLModel(ctx, provider).delete_actuals(model_id, 
        with_predictions=with_predictions, begin_date=begin_date, end_date=end_date, locally=locally)

@click.command('get_info', short_help='Get information about model.')
@click.argument('model-id', required=True, type=click.STRING)
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
help='Cloud AutoML Provider.')
@click.option('--locally', is_flag=True, default=False,
    help='Process actuals locally.')
@pass_context
def get_info(ctx, model_id, provider, locally):
    result = A2MLModel(ctx, provider).get_info(model_id, locally=locally)

    ctx.log('Model information for %s: %s' % (model_id, result))

@pass_context
def add_commands(ctx):
    cmdl.add_command(deploy)
    cmdl.add_command(predict)
    cmdl.add_command(actuals)
    cmdl.add_command(review_alert)
    cmdl.add_command(review)
    cmdl.add_command(undeploy)
    cmdl.add_command(delete_actuals)
    cmdl.add_command(get_info)

add_commands()



import click
from a2ml.api.a2ml_dataset import A2MLDataset
from a2ml.api.utils.context import pass_context


@click.group('dataset', short_help='Dataset(s) management')
@pass_context
def cmdl(ctx):
    """Dataset(s) management"""
    ctx.setup_logger(format='')


@click.command(short_help='List Data Sets on Provider Cloud')
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
    help='Cloud AutoML Provider.')
@pass_context
def list_cmd(ctx, provider):
    """List Provider remote datasets"""
    A2MLDataset(ctx, provider).list()


@click.command(short_help='Create Dataset on Provider Cloud')
@click.argument('source', required=False, type=click.STRING)
@click.option('--description', '-d', type=click.STRING, required=False,
    help='Description of dataset.')
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
    help='Cloud AutoML Provider.')
@pass_context
def create(ctx, source, description, provider):
    """Create data set on the Provider Cloud.
       If source is not specified, config.yaml/source
       will be used instead.
    """
    A2MLDataset(ctx, provider).create(source, description=description)


@click.command(short_help='Delete Dataset on Provider Cloud')
@click.argument('name', required=False, type=click.STRING)
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
    help='Cloud AutoML Provider.')
@pass_context
def delete(ctx, provider, name):
    """Delete Dataset on the Provider Cloud
       If name is not specified, config.yaml/dataset
       will be used instead.
    """
    A2MLDataset(ctx, provider).delete(name)


@click.command(short_help='Select Dataset')
@click.argument('name', required=False, type=click.STRING)
@click.option('--provider', '-p', type=click.Choice(['auger','azure']), required=False,
    help='Cloud AutoML Provider.')
@pass_context
def select(ctx, provider, name):
    """Select data set.
       Name will be set in config.yaml/dataset
    """
    A2MLDataset(ctx, provider).select(name)


@pass_context
def add_commands(ctx):
    cmdl.add_command(list_cmd, name='list')
    cmdl.add_command(create)
    cmdl.add_command(delete)
    cmdl.add_command(select)


add_commands()



import os

import pytest

from .utils import interceptor, ORGANIZATIONS, PROJECTS, PROJECT_FILES, PROJECT_FILE, EXPERIMENTS
from a2ml.api.auger.experiment import AugerExperiment


EXPERIMENT_SESSION = {
    'data': {
        'id': 'test_id_2',
        'model_settings': {'start_time': '2019-06-28 20:30:00.992405'},
        'status': 'completed',
        'project_file_id': 1256,
    }
}

EXPERIMENT_SESSIONS = {
    'meta': {
        'pagination': {'offset': 0, 'count': 2, 'total': 2, 'limit': 100},
        'status': 200},
    'data': [{
        'id': 'test_id_1',
        'model_settings': {'start_time': '2019-06-26 22:00:00.405'},
        'status': 'completed',
    },
    EXPERIMENT_SESSION['data']
    ]
}

TRIALS = {
    'meta': {'pagination': {'offset': 0, 'limit': 100, 'count': 20, 'total': 20}, 'status': 200},
    'data': [{
        'id': 'A79FBADD8CCD417',
        'score_name': 'f1_macro',
        'score_value': 0.123,
        'hyperparameter': {'algorithm_name': 'auger_ml.algorithms.baseline.BaselineClassifier'},
    }]*20
}

PROJECT = {
    'data': {
        'status': 'running',
    }
}


class TestExperiment():
    def test_list(self, log, project, ctx, authenticated, monkeypatch):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'get_projects': PROJECTS,
            'get_project': PROJECT,
            'get_project_files': PROJECT_FILES,
            'get_experiments': EXPERIMENTS,
        }
        interceptor(PAYLOAD, monkeypatch)

        result = AugerExperiment(ctx).list()

        assert result.get('experiments')
        assert len(result['experiments']) == 1
        assert result['experiments'][0]['name'] == 'iris-1.csv-experiment'
        assert result['experiments'][0]['project_file_id'] == 1256

    def test_start(self, log, project, ctx, authenticated, monkeypatch):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'get_projects': PROJECTS,
            'get_project': PROJECT,
            'get_project_files': PROJECT_FILES,
            'get_project_file': PROJECT_FILE,
            'update_project': PROJECT,
            'get_experiments': EXPERIMENTS,
            'get_experiment_sessions': EXPERIMENT_SESSIONS,
            'create_experiment_session': EXPERIMENT_SESSION,
            'get_experiment_session': EXPERIMENT_SESSION,
            'update_experiment_session': EXPERIMENT_SESSION,
            'get_trials': TRIALS,
        }
        interceptor(PAYLOAD, monkeypatch)

        monkeypatch.setattr('a2ml.api.auger.impl.cloud.base.AugerBaseApi.status', lambda *a, **kw: 'started')

        result = AugerExperiment(ctx).start()

        assert result.get('experiment_name') == 'iris-1.csv-experiment'
        assert result.get('session_id') == 'test_id_2'

    @pytest.mark.skip(reason="Make it work first, edge cases next")
    def test_start_without_target(self, log, project, ctx, authenticated, monkeypatch):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'get_projects': PROJECTS,
            'get_project': PROJECT,
            'get_project_files': PROJECT_FILES,
            'get_experiments': EXPERIMENTS,
            'get_experiment_sessions': EXPERIMENT_SESSIONS,
            'get_trials': TRIALS,
        }
        interceptor(PAYLOAD, monkeypatch)
        # TODO: ensure cli throws error on trying to start exp w/o target
        result = AugerExperiment(ctx).start()
        # assert result.exit_code != 0
        # assert log.messages[-1] == 'Please set target to build model.'

    def test_stop(self, log, project, ctx, authenticated, monkeypatch):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'get_projects': PROJECTS,
            'get_project_files': PROJECT_FILES,
            'get_experiments': EXPERIMENTS,
            'get_experiment_sessions': EXPERIMENT_SESSIONS,
            'update_experiment_session': EXPERIMENT_SESSION,
        }
        interceptor(PAYLOAD, monkeypatch)
        monkeypatch.setattr('a2ml.api.auger.impl.cloud.experiment_session.AugerExperimentSessionApi.status', lambda *a, **kw: 'started')

        result = AugerExperiment(ctx).stop()
        assert result.get('stopped') == 'iris-1.csv-experiment'
        # assert result.exit_code == 0
        # assert log.messages[0] == 'Search is stopped...'

    def test_leaderboard(self, log, project, ctx, authenticated, monkeypatch):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'get_projects': PROJECTS,
            'get_project_files': PROJECT_FILES,
            'get_experiments': EXPERIMENTS,
            'get_experiment_session': EXPERIMENT_SESSION,
            'get_experiment_sessions': EXPERIMENT_SESSIONS,
            'get_trials': TRIALS,
        }
        interceptor(PAYLOAD, monkeypatch)

        result = AugerExperiment(ctx).leaderboard()

        assert result.get('leaderboard')
        assert len(result.get('leaderboard')) == 10
        assert result.get('status') == 'completed'

    def test_history(self, log, project, ctx, authenticated, monkeypatch):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'get_projects': PROJECTS,
            'get_project_files': PROJECT_FILES,
            'get_experiments': EXPERIMENTS,
            'get_experiment_sessions': EXPERIMENT_SESSIONS,
        }
        interceptor(PAYLOAD, monkeypatch)

        result = AugerExperiment(ctx).history()
        assert result.get('history')
        assert len(result.get('history')) == 2
        assert result['history'][0].get('status') == 'completed'




from .base import AugerBaseApi
from ..exceptions import AugerException


class AugerReviewAlertItemApi(AugerBaseApi):
    """Auger Review Alert Item API."""

    def __init__(self, ctx, endpoint_api):
        super(AugerReviewAlertItemApi, self).__init__(ctx, endpoint_api)



from .base import AugerBaseApi
from ..exceptions import AugerException


class AugerTrialApi(AugerBaseApi):
    """Auger Trial API."""

    def __init__(
        self, ctx, experiment_session_api,
        trial_name=None, trial_id=None):
        super(AugerTrialApi, self).__init__(
            ctx, experiment_session_api, trial_name, trial_id)



import os
import csv

from ..cloud.pipeline import AugerPipelineApi

class ModelDeleteActual():
    """Predict using deployed Auger Model."""

    def __init__(self, ctx):
        super(ModelDeleteActual, self).__init__()
        self.ctx = ctx

    def execute(self, model_id, with_predictions, begin_date, end_date):
        pipeline_api = AugerPipelineApi(self.ctx, None, model_id)

        return pipeline_api.delete_actuals(with_predictions, begin_date, end_date)



import pandas as pd

from .interpreter import Interpreter
from .lexer import Lexer
from .parser import Parser
from .validator import Validator
from .var_names_fetcher import VarNamesFetcher


class Calculator:
    def __init__(self, revenue=None, investment=None, filter=None, known_vars=[], vars_mapping={}):
        self.revenue = revenue
        self.investment = investment
        self.filter = filter
        self.known_vars = known_vars + list(vars_mapping.keys())
        self.vars_mapping = vars_mapping

        self.revenue_interpreter = self.build_interpreter(self.revenue)
        self.investment_interpreter = self.build_interpreter(self.investment)
        self.filter_interpreter = self.build_interpreter(self.filter)

    def build_interpreter(self, expression):
        if expression:
            return Interpreter(expression, self.vars_mapping)

    def calculate(self, rows):
        if isinstance(rows, pd.DataFrame):
            rows = list(map(lambda x: x[1].to_dict(), rows.iterrows()))

        filtered_rows = rows

        if self.filter_interpreter:
            filtered_rows = self.filter_interpreter.run(rows, filter=True)

        if len(filtered_rows) > 0:
            revenue = sum(self.revenue_interpreter.run(filtered_rows))
            investment = sum(self.investment_interpreter.run(filtered_rows))
        else:
            revenue = 0
            investment = 0

        if investment > 0:
            roi = (revenue - investment) / investment
        else:
            roi = 0

        return {
            "count": len(filtered_rows),
            "filtered_rows": filtered_rows,
            "revenue": revenue,
            "investment": investment,
            "roi": roi,
        }

    def get_var_names(self):
        result = []
        if self.revenue:
            fetcher = VarNamesFetcher(self.revenue)
            result += fetcher.fetch()

        if self.investment:
            fetcher = VarNamesFetcher(self.investment)
            result += fetcher.fetch()

        if self.filter:
            fetcher = VarNamesFetcher(self.filter)
            result += fetcher.fetch()

        result = set(result)    
        return [var[1:] for var in result if var.startswith('$')]



import os
import pytest

from .utils import interceptor, object_status_chain, ORGANIZATIONS, PROJECTS
from a2ml.api.auger.project import AugerProject

class TestProject():
    def test_list(self, log, ctx, monkeypatch, authenticated):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'get_projects': PROJECTS,
        }
        interceptor(PAYLOAD, monkeypatch)

        result = AugerProject(ctx).list()

        assert result.get('projects')
        assert len(result['projects']) == 2
        assert result['projects'][0]['name'] == 'project_1'

    def test_create(self, log, ctx, authenticated, monkeypatch):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'create_project': {
                'data': {
                    'id': 1,
                    'name': 'igor-test',
                },
            },
            'get_project': {
                'data': {
                    'id': 1,
                    'name': 'igor-test',
                },
            }
        }
        interceptor(PAYLOAD, monkeypatch)

        result = AugerProject(ctx).create('igor-test')
        assert result.get('created') == 'igor-test'

    def test_delete(self, log, project, ctx, authenticated, monkeypatch):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'get_projects': PROJECTS,
            'delete_project': {
                'data': {}
            }
        }
        interceptor(PAYLOAD, monkeypatch)

        result = AugerProject(ctx).delete('project_1')
        assert result.get('deleted') == 'project_1'

    def test_select(self, log, project, ctx, authenticated):
        assert ctx.config.get('name', '') == 'cli-integration-test'

        result = AugerProject(ctx).select('another_project')

        assert ctx.config.get('name', '') == 'another_project'
        assert result.get('selected') == 'another_project'

    def test_start(self, log, project, ctx, authenticated, monkeypatch):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'get_organization': {'data': {} },
            'get_projects': PROJECTS,
            'get_project': {'data': {'status': 'running'}},
            'update_project': {'data': {}},
            # 'deploy_project': {'data': {}}
        }
        interceptor(PAYLOAD, monkeypatch)
        object_status_chain(['deploying', 'deployed', 'running'], monkeypatch)

        result = AugerProject(ctx).start("cli-integration-test")
        assert result.get('running') == 'cli-integration-test'

    def test_stop(self, log, project, ctx, authenticated, monkeypatch):
        PAYLOAD = {
            'get_organizations': ORGANIZATIONS,
            'get_projects': PROJECTS,
            'get_project': {
                'data': {
                    'id': 1,
                    'name': 'project_1',
                    'status': 'running',
                },
                'meta': {'status': 404},
            },
            'undeploy_project': {'meta': {'status': 200}, 'data': {}},
        }
        interceptor(PAYLOAD, monkeypatch)
        object_status_chain(['running', 'undeploying', 'undeployed'], monkeypatch)
        monkeypatch.setattr('a2ml.api.auger.impl.cloud.project.AugerProjectApi.status', lambda x: 'undeployed')

        result = AugerProject(ctx).stop("project_1")
        print(result)
        assert result.get('stopped') == 'project_1'
